<?php
/*
 * Copyright 2007-2015 Abstrium SAS <team (at) pyd.io>
 * This file is part of the Pydio Enterprise Distribution.
 * It is subject to the End User License Agreement that you should have
 * received and accepted along with this distribution.
 */

namespace Pydio\Action\Migration;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Pydio\Auth\Driver\LdapAuthDriver;
use Pydio\Auth\Driver\MultiAuthDriver;
use Pydio\Conf\Core\AbstractUser;
use Pydio\Core\Http\Response\SerializableResponseStream;
use Pydio\Core\Model\ContextInterface;
use Pydio\Core\PluginFramework\Plugin;
use Pydio\Core\PluginFramework\PluginsService;
use Pydio\Core\Services\AuthService;
use Pydio\Core\Services\ConfService;
use Pydio\Core\Services\RolesService;
use Pydio\Core\Services\UsersService;
use Zend\Diactoros\Response\JsonResponse;


defined('AJXP_EXEC') or die('Access not allowed');
require_once(__DIR__ . '/../vendor/autoload.php');
/**
 * Class SysSignature
 * @package Pydio\Action\Migration
 */
class Migration extends Plugin
{
    public function getMigrationActions(ServerRequestInterface $requestInterface, ResponseInterface &$responseInterface) {
        $ret = [];
        $action     = $requestInterface->getAttribute("action");
        /** @var ContextInterface $ctx */
        $ctx        = $requestInterface->getAttribute("ctx");
        $httpVars   = $requestInterface->getParsedBody();
        $loggedUser = $ctx->getUser();
        if ($loggedUser == null || !$loggedUser->isAdmin()) {
            return $responseInterface->withBody(new SerializableResponseStream(["error"=>"unauthorized"]));
        }

        $responseInterface = new JsonResponse(["error"=>"user not found"]);
        switch ($action) {
            case 'hashedpassword':
                $userId = $httpVars["userid"];
                // Check existance
                if (!UsersService::userExists($userId)){
                    return;
                }
                // Get auth schema
                $authDriver = ConfService::getAuthDriverImpl();
                $authSchema = $authDriver->getAuthScheme($userId);
                $userTmp = UsersService::getUserById($userId);
                $profile = $userTmp->getProfile($userId);
                if (empty($authSchema)) $authSchema = 'sql';
                $ret["login"] = $userId;
                $ret["authsource"] = $authSchema;
                $ret["profile"] = $profile;
                if($userTmp->hasParent()){
                    $ret["parent"] = $userTmp->getParent();
                }
                // Get hashed password
                if ($authSchema == 'sql') {
                    $dbret = \dibi::query("SELECT [password] from [ajxp_users] WHERE login = %s", $userId);
                    $userRow = $dbret->fetchSingle();
                    if (!empty($userRow)){
                        $ret["password"] = $userRow;
                    }
                }elseif($authSchema == 'ldap'){
                    // get DomainName
                    $ldapDriver = $authDriver->drivers['ldap'];
                    $ldapVisitor = new LdapVisitor($ldapDriver);
                    $domainName = $ldapVisitor->getDomainName();
                    $ret["authsource"] = strtolower(trim(preg_replace('/[^A-Za-z0-9-]+/', '-', $domainName)));;
                    $ret["domainname"] = $domainName;

                }
                $responseInterface = new JsonResponse($ret);

                break;
            case 'ldapdomainname':
                $authDriver = ConfService::getAuthDriverImpl();
                if($authDriver instanceof MultiAuthDriver && isset($authDriver->drivers['ldap'])) {
                    $ldapDriver = $authDriver->drivers['ldap'];
                    $ldapVisitor = new LdapVisitor($ldapDriver);
                    $responseInterface = new JsonResponse(['domainname' => $ldapVisitor->getNamingContext()]);
                } else{
                    $responseInterface = new JsonResponse(['domainname' => '']);
                }
                break;
            case 'listroles':
                $responseInterface = $this->listRoles($requestInterface);
                break;
            case 'getmetadata':
                if (empty($this->getMetadata())){
                    $responseInterface = new JsonResponse(new \stdClass());
                }
                else {
                    $responseInterface = new JsonResponse($this->getMetadata());
                }
                break;
            default:
        }
    }

    public function listRoles(ServerRequestInterface $requestInterface)
    {
        $httpVars   = $requestInterface->getParsedBody();
        $teams = $httpVars["teams"] === "true";
        $roles      = RolesService::getRolesList(array(), true, $teams);
        ksort($roles);
        $result = [];

        foreach ($roles as $roleObject) {
            $owner = $roleObject->getOwner();
            if ($teams && empty($owner)){
                continue;
            }
            $appliesToDefault = implode(",", $roleObject->listAutoApplies());
            $override = $roleObject->filterParameterValue("core.conf", "ROLE_FORCE_OVERRIDE", AJXP_REPO_SCOPE_ALL, false);
            $result[$roleObject->getId()]= [
                "role_id"           => $roleObject->getId(),
                "role_label"        => $roleObject->getLabel(),
                "applies_to"        => $appliesToDefault,
                "owner_id"          => empty($owner)?"": $owner,
                "force_override"    => $override,
                "role"              => $roleObject->getDataArray(true),
            ];
        }
        if(!count($result)){
            return new JsonResponse(new \stdClass());
        } else{
            return new JsonResponse($result);
        }

    }

    public function getMetadata(){
        $returnData = [];

        $metaStoreDir = AJXP_DATA_PATH."/plugins/metastore.serial";
        $metastores = glob($metaStoreDir."/ajxp_meta_*");


        foreach($metastores as $store){
            if(strpos($store, ".bak") !== false) continue;
            $fileName = basename($store);
            $temp = explode('_', $fileName);

            $data = unserialize(file_get_contents($store));
            foreach($data as $filePath => &$metadata){

                // Normal watch
                $watchList = [];
                if(isSet($metadata['AJXP_METADATA_SHAREDUSER']['META_WATCH'])){
                    $watchList = array_merge($watchList, $metadata['AJXP_METADATA_SHAREDUSER']['META_WATCH']);
                }

                if (($temp[2] == "07184f0f8e667257ada1c49aa9adccea") || ($temp[2] == "1")){
                    $test = "";
                }

                if(!empty($watchList)){
                if (isset($temp[2])) {
                    if (isset($temp[3])) {
                        $returnData[$temp[2]][$temp[3]][$filePath]['WATCH'] = $watchList;
                    }
                    else{
                        $returnData[$temp[2]]['AJXP_METADATA_SHAREDUSER'][$filePath]['WATCH'] = $watchList;
                    }
                }
                }

                // watch on Share
                $watchOnShareList = [];
                if(isSet($metadata['AJXP_METADATA_SHAREDUSER']['META_WATCH_USERS']) &&
                    isSet($metadata['AJXP_METADATA_SHAREDUSER']['META_WATCH_USERS']['META_WATCH_USERS_CHANGE'])){
                    $watchOnShareList = $metadata['AJXP_METADATA_SHAREDUSER']['META_WATCH_USERS']['META_WATCH_USERS_CHANGE'];
                }

                if (!empty($watchOnShareList)){
                if (isset($temp[2])) {
                    if (isset($temp[3])) {
                        $returnData[$temp[2]][$temp[3]][$filePath]['META_WATCH_USERS'] = $watchOnShareList;
                    }
                    else{
                        $returnData[$temp[2]]['AJXP_METADATA_SHAREDUSER'][$filePath]['META_WATCH_USERS'] = $watchOnShareList;
                    }
                }
                }

                // bookmark
                foreach ($metadata as $key => $value ) {
                    if ($key != 'AJXP_METADATA_SHAREDUSER') {
                        if (isset($metadata[$key]['ajxp_bookmarked']) && $metadata[$key]['ajxp_bookmarked']['ajxp_bookmarked'] === "true"){
                            if (isset($temp[2])) {
                                if (isset($temp[3])) {
                                    $returnData[$temp[2]][$temp[3]][$filePath]['BOOKMARK'][$temp[2]] = true;
                                }
                                else{
                                    $returnData[$temp[2]]['AJXP_METADATA_SHAREDUSER'][$filePath]['BOOKMARK'][$key] = true;
                                }
                            }
                        }
                    }

                }
            }
        }

        return $returnData;
    }

    public function getUserProfile($userId){
        $user = UsersService::getUserById($userId);
        if ($user instanceof AbstractUser){
            return $user->getProfile();
        }
        return "";
    }
}


class LdapVisitor{
    public $ldap;
    public function __construct($ldapObject){
        $this->ldap = $ldapObject;
    }
    public function getDomainName(){
        $this->ldap->startConnexion();
        $ret = ldap_read($this->ldap->ldapconn, '', '(objectClass=*)', ['namingContexts']);
        if(!empty($ret)){
            $info = ldap_get_entries($this->ldap->ldapconn, $ret);
            if (!empty($info)){
                if(!empty($info[0]['namingcontexts'])){
                    $domainName = $info[0]['namingcontexts'][0];
                }
                $segments = explode(',', $domainName);
                $domainArray = [];
                foreach($segments as $segment ){
                    list($dc, $segmentName) = explode('=', $segment);
                    $domainArray[] = $segmentName;
                }
                return implode('.', $domainArray);
            }
        }
        return '';
    }

    public function getNamingContext(){
        $this->ldap->startConnexion();
        $ret = ldap_read($this->ldap->ldapconn, '', '(objectClass=*)', ['namingContexts']);
        if(!empty($ret)){
            $info = ldap_get_entries($this->ldap->ldapconn, $ret);
            if (!empty($info)){
                if(!empty($info[0]['namingcontexts'])){
                    return $info[0]['namingcontexts'][0];
                }
            }
        }
        return '';
    }
}
